// --------------------------------------------------------------------------
// Drawing.java
// A simple interface for drawing pictures in Java.
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.jku.at>
// Created: 7.6.2013; Last modified: 7.6.2013
// --------------------------------------------------------------------------
package kwm;

import javax.swing.*; 
import java.awt.*;

public class Drawing extends JPanel
{
  private static final long serialVersionUID = 1L;

  // ------------------------------------------------------------------------
  // a small test program demonstrating the usage
  // ------------------------------------------------------------------------
  public static void main(String[] args)
  {
    Drawing.begin("HelloWorld #1", 300, 200);
    Drawing.graphics.drawRect(40, 60, 100, 100);
    Drawing.end();

    Drawing.begin("HelloWorld #2");
    Drawing.graphics.drawOval(40, 60, 100, 100);
    for (int i=0; i<400; i++)
    {
      for (int j=0; j<400; j++)
        Drawing.graphics.drawLine(i, j, i, j);
      Drawing.paint();
    }
    Drawing.end();
  }

  // the current frame and graphics to draw to
  public static JFrame frame;
  public static JPanel panel;
  public static Graphics graphics;

  // ------------------------------------------------------------------------
  // create a frame width denoted title. There may be up to four additional
  // parameters width, height, iwidth, iheight denoting the initial size of 
  // of the frame (width, height) and the size of the image displayed
  // in it (iwidth, iheight).
  // ------------------------------------------------------------------------
  public static void begin(final String title, int... params)
  {
    if (frame != null)
      throw new RuntimeException("Drawing.begin() was called when window was " +
          " still open (call Drawing.end() before)");
    try
    {
      int n = params.length;
      final int width   = n < 1 ? 640  : params[0]; 
      final int height  = n < 2 ? 480  : params[1];
      final int iwidth  = n < 3 ? 1920 : params[1];
      final int iheight = n < 4 ? 1080 : params[1];
      SwingUtilities.invokeAndWait(new Runnable() {
        public void run() {
          frame = new JFrame(title);
          frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
          Drawing drawing = new Drawing(width, height);
          panel = drawing;
          frame.add(drawing); 
          frame.pack();
          drawing.makeImage(iwidth, iheight);
          frame.setVisible(true);
        }
      });
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  // ------------------------------------------------------------------------
  // close the current frame, allows to open another one
  // ------------------------------------------------------------------------
  public static void end()
  {
    if (frame == null)
    {
      throw new RuntimeException("Drawing.end() was called when no window was " +
          " open any more (call Drawing.begin() before)");
    }
    paint();
    System.out.println("Press <ENTER> to continue or close window to quit...");
    Input.readString();
    frame.dispose();
    frame = null;
  }

  // ------------------------------------------------------------------------
  // make sure all of the painting is displayed
  // ------------------------------------------------------------------------
  public static void paint()
  {
    frame.repaint();
  }

  // the width and height of the drawing panel 
  private int width;
  private int height;

  // the image from which the graphics is derived
  private Image image;

  // ------------------------------------------------------------------------
  // create a panel of denoted width and height
  // ------------------------------------------------------------------------
  private Drawing(int width, int height)
  {
    this.width = width;
    this.height = height;
  }

  // ------------------------------------------------------------------------
  // tell preferred size such that frame has appropriate size upon opening
  // ------------------------------------------------------------------------
  public Dimension getPreferredSize() 
  {  
    return new Dimension(width, height); 
  }

  // ------------------------------------------------------------------------
  // associate an image of denoted width and height to the panel
  // must be called *after* panel has been added to frame
  // (otherwise createImage() returns null)
  // ------------------------------------------------------------------------
  public void makeImage(int width, int height) 
  { 
    image = createImage(width, height); 
    graphics = image.getGraphics();
    graphics.setColor(Color.WHITE);
    graphics.fillRect(0, 0, width, height);
    graphics.setColor(Color.BLACK);
  }

  // ------------------------------------------------------------------------
  // paint the image
  // ------------------------------------------------------------------------
  public void paint(Graphics graphics)
  {
    graphics.drawImage(image, 0, 0, this);
  }
}
